import {CommandModule} from "yargs";
import {inquire} from "../prompts/Inquire";
import {registerPrompts} from "../prompts/types";
import {LocalRepository} from "../repository/LocalRepository";
import {empathyGroupPrompt} from "../prompts/implementations/EmpathyGroupPrompt";
import {empathyGroupListPrompt} from "../prompts/implementations/EmpathyGroupListPrompt";
import {empathyGuidePrompt} from "../prompts/implementations/EmpathyGuidePrompt";

export function updateEmpathyGuideCommand(): CommandModule {
    return {
        command: "update-empathy-guide",
        describe: "Makes changes to the empathy guide.",
        handler: async (): Promise<void> => {
            registerPrompts()
            const storage = new LocalRepository()
            const empathyGuide = await storage.loadEmpathyGuide()

            const empathyGroup = empathyGroupPrompt({
                inquire,
                showError: async (text) => console.error(text),
            })

            const empathyList = empathyGroupListPrompt({
                inquire,
                promptForEmpathyGroup: empathyGroup
            })

            const newGuide = await empathyGuidePrompt({
                inquire,
                showError: async (text) => console.error(text),
                promptForEmpathyGroupList: empathyList,
            })({default: empathyGuide})
            await storage.saveEmpathyGuide(newGuide)
            console.log("Empathy guide saved!")
        }
    }
}